import unittest

from src.Runner import match

""" Test Class, feel free to add more tests or run using the CLI"""


class MatchSoftwareEngineerTest(unittest.TestCase):
    @staticmethod
    def test_automation_engineer():
        match(['-t', 'engineer', '-s', 'java', '-s', 'selenium'])

    @staticmethod
    def test_software_engineer():
        match(['-t', 'software'])

    @staticmethod
    def test_no_skills():
        match(['-t', 'engineer'])

    @staticmethod
    def test_automation_engineer_no_match():
        """ This test will not find a match since there is no Automation Engineer with Hadoop in it's skills"""
        match(['-t', 'engineer', '-s', 'java', '-s', 'selenium', '-s', 'hadoop'])


if __name__ == '__main__':
    unittest.main()



from typing import Optional

from src.Skills import Skills


class Job:

    def __init__(self, title, skills: Optional[str]):
        self.title = title
        self.skills = Skills(skills_list=skills)





from typing import Optional

from src.Skills import Skills


class Candidate:
    def __init__(self, name: str, title: str, skills: Optional[str]):
        self.skills = Skills(skills_list=skills)
        self.name = name
        self.title = title

    def __str__(self):
        return f'------------------\nCandidate\n------------------\nName: {self.name}\nTitle: {self.title}\nSkills:' \
               f'\n{self.skills}'



import json

from pkg_resources import resource_stream as rs

from src.Job import Job
from src.Candidate import Candidate


def run_matcher(job: Job) -> [Candidate]:
    """
    search matching Candidate for given Job
    :param job: Job object for search criteria
    """
    candidates_list = __extract_json()['candidates']
    job_skills_list = job.skills.get_list()
    qualified_candidates = list(filter(lambda candidate: job.title.lower() in candidate['title'].lower()
                                and set(job_skills_list).issubset(set(map(str.lower, candidate['skills']))),
                                candidates_list))
    return __build_candidates(qualified_candidates=qualified_candidates)


def __extract_json():
    """ Private method for extracting 'Database' JSON"""
    json_string = rs("src.resources", "database.json").read().decode()
    loaded_json = json.loads(json_string)
    return loaded_json


def __build_candidates(qualified_candidates):
    """ Build Candidate object per dict object from Database"""
    return [Candidate(**candidate) for candidate in qualified_candidates]



import click
from src.Job import Job
from src.CandidateFinder import run_matcher


@click.command(context_settings=dict(help_option_names=['-h', '--help']))
@click.option('--title', '-t', required=True, prompt=True, help='The job title')
@click.option('--skills', '-s', required=False, multiple=True, default=[],
              help='Optional Skills for the job, you can add multiple skills by using -s/-skill')
def match(title, skills):
    """Returns the best matches for the provided Job"""
    job = Job(title=title, skills=skills)
    print(*run_matcher(job=job), sep='')



from setuptools import setup, find_packages

setup(
    name='gloat-matcher',
    version='0.2',
    description='Backend assigment for Gloat',
    packages=find_packages(include=['src', 'src.*']),
    package_data={'': ['database.json']},
    download_url='https://github.com/Rantzur1992/gloat-backend/archive/0.1.tar.gz',
    python_requires='>3.7',
    install_requires=[
        'Click'
    ],
    entry_points=
    '''
        [console_scripts]
        matcher=src.Runner:match
    ''',
)



class Skill:

    def __init__(self, name: str):
        self.name = name

    def __str__(self):
        return self.name


class Skills:
    def __init__(self, skills_list: [str]):
        self.__skills = list(map(lambda x: Skill(x), skills_list))

    def __str__(self):
        res = ""
        for skill in self.__skills:
            res += f'   - {skill}\n'
        return res

    def get_list(self):
        return list(map(lambda x: x.name.lower(), self.__skills))
